@@ -38,6 +38,7 @@ class Serial(COMM):
             getConfig
     '''
     id = "serial"
+    name = _("Serial")
     showSerialComboboxSignal = pyqtSignal(list)
     showSwitchSignal = pyqtSignal(ConnectionStatus)
     def onInit(self, config):
@@ -73,6 +74,14 @@ class Serial(COMM):
         self.widget = None
         self.baudrateCustomStr = _("Custom, input baudrate")
 
+    def disconnect(self):
+        if self.isConnected():
+            self.openCloseSerial()
+
+    def onDel(self):
+        if self.isConnected():
+            self.openCloseSerial()
+
     def __del__(self):
         try:
             self.com.close()
@@ -93,7 +102,7 @@ class Serial(COMM):
         self.detectSerialPort()
 
     def onWidget(self):
-        serialSetting = QWidget()
+        self.widget = QWidget()
         serialSettingsLayout = QGridLayout()
         serialPortLabek = QLabel(_("Port"))
         serailBaudrateLabel = QLabel(_("Baudrate"))
@@ -152,7 +161,7 @@ class Serial(COMM):
         serialSettingsLayout.addWidget(self.checkBoxRTS, 6, 0,1,1)
         serialSettingsLayout.addWidget(self.checkBoxDTR, 6, 1,1,1)
         serialSettingsLayout.addWidget(self.serialOpenCloseButton, 7, 0,1,2)
-        serialSetting.setLayout(serialSettingsLayout)
+        self.widget.setLayout(serialSettingsLayout)
         self.widgetConfMap["port"]       = self.serialPortCombobox
         self.widgetConfMap["baudrate"]    = self.serailBaudrateCombobox
         self.widgetConfMap["bytesize"]    = self.serailBytesCombobox
@@ -162,8 +171,7 @@ class Serial(COMM):
         self.widgetConfMap["rts"]         = self.checkBoxRTS
         self.widgetConfMap["dtr"]         = self.checkBoxDTR
         self.initEvet()
-        self.widget = serialSetting
-        return serialSetting
+        return self.widget
 
     def initEvet(self):
         self.serialPortCombobox.clicked.connect(self.detectSerialPort)
@@ -254,6 +262,8 @@ class Serial(COMM):
             elif conf_type == "stopbits":
                 value = float(value)
             self.com.__setattr__(conf_type, value)
+            if conf_type == "baudrate":
+                self.oneByteTime = 1 / (self.com.baudrate / (self.com.bytesize + 2 + self.com.stopbits)) # 1 byte use time
         elif conf_type == "flowcontrol":
             values = getCommboboxItems(obj)
             idx = 0
@@ -304,6 +314,7 @@ class Serial(COMM):
         else:
             try:
                 print("-- open serial")
+                self.onConnectionStatus.emit(ConnectionStatus.CONNECTING, "")
                 self.com.open()
                 self.status = ConnectionStatus.CONNECTED
                 self.onConnectionStatus.emit(self.status, "")
@@ -410,6 +421,7 @@ class Serial(COMM):
             if waitingReconnect:
                 if self.portExits(self.com.port):
                     try:
+                        self.onConnectionStatus.emit(ConnectionStatus.CONNECTING, "")
                         self.com.open()
                         print("-- reopen serial")
                         waitingReconnect = False
@@ -429,7 +441,7 @@ class Serial(COMM):
                         buffer += data
                         continue
                     buffer += data
-                if buffer and (time.time() - t > 0.001): # no new data in 1ms
+                if buffer and (time.time() - t > self.oneByteTime * 2): # no new data in next frame
                     try:
                         self.onReceived(buffer)
                     except Exception as e:
