@@ -1,10 +1,10 @@
 from . import dbg
 from . import protocol
 from .import terminal
-from . import gragh
+from . import graph
 # from . import myplugin
 
-pluginClasses = [dbg.Plugin, protocol.Plugin, terminal.Plugin, gragh.Plugin]
+pluginClasses = [dbg.Plugin, protocol.Plugin, terminal.Plugin, graph.Plugin]
 # pluginClasses.append(myplugin.Plugin)
 
 builtinPlugins = {}
