@@ -90,7 +90,10 @@ class Plugin(Plugin_Base):
             "showTimestamp" : False,
             "recordSend" : False,
             "saveLogPath" : "",
+            "saveLogPath2" : "",
             "saveLog" : False,
+            "wrap": False,
+            "saveLogAutoNew": False,
             "color" : False,
             "sendEscape" : False,
             "customSendItems" : [],
@@ -99,6 +102,8 @@ class Plugin(Plugin_Base):
         for k in default:
             if not k in self.config:
                 self.config[k] = default[k]
+        self.lastShowTail = ''
+        self.justSent = False # sent data before received data flag
 
     def onWidgetMain(self, parent):
         self.mainWidget = QSplitter(Qt.Vertical)
@@ -106,9 +111,7 @@ class Plugin(Plugin_Base):
         self.receiveArea = QTextEdit()
         font = QFont('Menlo,Consolas,Bitstream Vera Sans Mono,Courier New,monospace, Microsoft YaHei', 10)
         self.receiveArea.setFont(font)
-        self.receiveArea.setLineWrapMode(QTextEdit.NoWrap)
         self.sendArea = QTextEdit()
-        self.sendArea.setLineWrapMode(QTextEdit.NoWrap)
         self.sendArea.setAcceptRichText(False)
         self.clearReceiveButtion = QPushButton("")
         utils_ui.setButtonIcon(self.clearReceiveButtion, "mdi6.broom")
@@ -160,12 +163,15 @@ class Plugin(Plugin_Base):
         self.receiveSettingsTimestamp.setToolTip(_("Add timestamp before received data, will automatically enable auto line feed"))
         self.receiveSettingsColor = QCheckBox(_("Color"))
         self.receiveSettingsColor.setToolTip(_("Enable unix terminal color support, e.g. \\33[31;43mhello\\33[0m"))
+        self.receiveSettingsWrap = QCheckBox(_("Display wrap"))
+        self.receiveSettingsWrap.setToolTip(_("When content in a line is too long, always auto wrap to show, and no scroll bar"))
         serialReceiveSettingsLayout.addWidget(self.receiveSettingsAscii,1,0,1,1)
         serialReceiveSettingsLayout.addWidget(self.receiveSettingsHex,1,1,1,1)
         serialReceiveSettingsLayout.addWidget(self.receiveSettingsAutoLinefeed, 2, 0, 1, 1)
         serialReceiveSettingsLayout.addWidget(self.receiveSettingsAutoLinefeedTime, 2, 1, 1, 1)
         serialReceiveSettingsLayout.addWidget(self.receiveSettingsTimestamp, 3, 0, 1, 1)
         serialReceiveSettingsLayout.addWidget(self.receiveSettingsColor, 3, 1, 1, 1)
+        serialReceiveSettingsLayout.addWidget(self.receiveSettingsWrap, 4, 0, 1, 1)
         serialReceiveSettingsGroupBox.setLayout(serialReceiveSettingsLayout)
         serialReceiveSettingsGroupBox.setAlignment(Qt.AlignHCenter)
         layout.addWidget(serialReceiveSettingsGroupBox)
@@ -224,6 +230,7 @@ class Plugin(Plugin_Base):
         self.receiveSettingsAutoLinefeedTime.textChanged.connect(lambda: self.bindVar(self.receiveSettingsAutoLinefeedTime, self.config, "receiveAutoLindefeedTime", vtype=int, vErrorMsg=_("Auto line feed value error, must be integer"), emptyDefault = "200"))
         self.sendSettingsScheduled.textChanged.connect(lambda: self.bindVar(self.sendSettingsScheduled, self.config, "sendScheduledTime", vtype=int, vErrorMsg=_("Timed send value error, must be integer"), emptyDefault = "300"))
         self.sendSettingsScheduledCheckBox.clicked.connect(lambda: self.bindVar(self.sendSettingsScheduledCheckBox, self.config, "sendScheduled"))
+        self.receiveSettingsWrap.clicked.connect(self.onSettingWrap)
         return widget
 
 
@@ -274,14 +281,19 @@ class Plugin(Plugin_Base):
         self.customSendScroll.setWidget(cutomSendItemsWraper)
         self.customSendScroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         #
+        logFileWrapper = QVBoxLayout()
         logFileLayout = QHBoxLayout()
         self.saveLogCheckbox = QCheckBox()
         self.logFilePath = QLineEdit()
         self.logFileBtn = QPushButton(_("Log path"))
+        self.saveLogAutoNew = QCheckBox(_("Auto new file"))
+        self.saveLogAutoNew.setToolTip(_("When start a new connection, will automatically create a new log file"))
         logFileLayout.addWidget(self.saveLogCheckbox)
         logFileLayout.addWidget(self.logFilePath)
         logFileLayout.addWidget(self.logFileBtn)
-        self.logFileGroupBox.setLayout(logFileLayout)
+        logFileWrapper.addLayout(logFileLayout)
+        logFileWrapper.addWidget(self.saveLogAutoNew)
+        self.logFileGroupBox.setLayout(logFileWrapper)
         sendFunctionalLayout.addWidget(self.logFileGroupBox)
         sendFunctionalLayout.addWidget(self.fileSendGroupBox)
         sendFunctionalLayout.addWidget(self.clearHistoryButton)
@@ -293,6 +305,7 @@ class Plugin(Plugin_Base):
         self.sendFileButton.clicked.connect(self.sendFile)
         self.saveLogCheckbox.clicked.connect(self.setSaveLog)
         self.logFileBtn.clicked.connect(self.selectLogFile)
+        self.saveLogAutoNew.clicked.connect(lambda: self.bindVar(self.saveLogAutoNew, self.config, "saveLogAutoNew"))
         self.openFileButton.clicked.connect(self.selectFile)
         self.addButton.clicked.connect(self.customSendAdd)
         self.clearHistoryButton.clicked.connect(self.clearHistory)
@@ -314,6 +327,7 @@ class Plugin(Plugin_Base):
             interval = parameters.Parameters.receiveAutoLindefeedTime
         self.receiveSettingsAutoLinefeedTime.setText(str(interval) if interval > 0 else str(parameters.Parameters.receiveAutoLindefeedTime))
         self.receiveSettingsTimestamp.setChecked(paramObj["showTimestamp"])
+        self.receiveSettingsWrap.setChecked(paramObj["wrap"])
         self.sendSettingsHex.setChecked(not paramObj["sendAscii"])
         self.sendSettingsScheduledCheckBox.setChecked(paramObj["sendScheduled"])
         try:
@@ -332,7 +346,11 @@ class Plugin(Plugin_Base):
         self.logFilePath.setText(paramObj["saveLogPath"])
         self.logFilePath.setToolTip(paramObj["saveLogPath"])
         self.saveLogCheckbox.setChecked(paramObj["saveLog"])
+        self.saveLogAutoNew.setChecked(paramObj["saveLogAutoNew"])
         self.receiveSettingsColor.setChecked(paramObj["color"])
+        # wrap
+        self.receiveArea.setLineWrapMode(QTextEdit.WidgetWidth if paramObj["wrap"] else QTextEdit.NoWrap)
+        self.sendArea.setLineWrapMode(QTextEdit.WidgetWidth if paramObj["wrap"] else QTextEdit.NoWrap)
         # send items
         for text in paramObj["customSendItems"]:
             self.insertSendItem(text, load=True)
@@ -378,6 +396,13 @@ class Plugin(Plugin_Base):
             self.config["receiveAutoLinefeed"] = True
             self.receiveSettingsAutoLinefeed.setChecked(True)
 
+    def onSettingWrap(self):
+        wrap = self.receiveSettingsWrap.isChecked()
+        self.config["wrap"] = wrap
+        flag = QTextEdit.WidgetWidth if wrap else QTextEdit.NoWrap
+        self.receiveArea.setLineWrapMode(flag)
+        self.sendArea.setLineWrapMode(flag)
+
     def onEscapeSendClicked(self):
         self.config["sendEscape"] = self.sendSettingsEscape.isChecked()
 
@@ -443,12 +468,30 @@ class Plugin(Plugin_Base):
         self.logFilePath.setText(fileName_choose)
         self.logFilePath.setToolTip(fileName_choose)
         self.config["saveLogPath"] = fileName_choose
+        self.config["saveLogPath2"] = fileName_choose
+
+    def updateLogPath(self):
+        '''
+            update log path add datetiem and com port name
+        '''
+        if not self.config["saveLogPath"]:
+            return
+        date = datetime.now().strftime("%Y_%m_%d_%H_%M_%S")
+        path = os.path.splitext(self.config['saveLogPath'])
+        self.config["saveLogPath2"] = f"{path[0]}_{date}{path[1]}"
+
 
     def onLog(self, text):
-        if self.config["saveLogPath"]:
-            with open(self.config["saveLogPath"], "a+", encoding=self.configGlobal["encoding"], newline="\n") as f:
+        path = self.config["saveLogPath2"] if self.config["saveLogAutoNew"] else self.config["saveLogPath"]
+        if self.config["saveLog"] and path:
+            with open(path, "a+", encoding=self.configGlobal["encoding"], newline="\n") as f:
                 f.write(text)
 
+    def onConnChanged(self, status:ConnectionStatus, msg:str):
+        super().onConnChanged(status, msg)
+        if status == ConnectionStatus.CONNECTED and self.config["saveLogAutoNew"]:
+            self.updateLogPath()
+
     def onKeyPressEvent(self, event):
         if event.key() == Qt.Key_Control:
             self.keyControlPressed = True
@@ -468,10 +511,19 @@ class Plugin(Plugin_Base):
 
     def insertSendItem(self, text="", load = False):
         itemsNum = self.customSendItemsLayout.count() + 1
+        # TODO: here auto set scroll area height is ugly, maybe have better way
         height = parameters.customSendItemHeight * (itemsNum + 1) + 20
         topHeight = self.fileSendGroupBox.height() + self.logFileGroupBox.height() + 100
-        if height + topHeight > self.funcParent.height():
-            height = self.funcParent.height() - topHeight
+        # print("1:", parameters.customSendItemHeight, itemsNum + 1, height, topHeight)
+        # print("2:", height + topHeight, self.funcParent.height())
+        # if height + topHeight > self.funcParent.height():
+        #     height = self.funcParent.height() - topHeight
+        # if height < 0:
+        #     height = self.funcParent.height() // 3
+        screenH = QApplication.desktop().screenGeometry().height()
+        screenH -= screenH // 3
+        if height + topHeight >= screenH:
+            height = screenH - topHeight
         if height < 0:
             height = self.funcParent.height() // 3
         self.customSendScroll.setMinimumHeight(height)
@@ -501,11 +553,16 @@ class Plugin(Plugin_Base):
     def deleteSendItem(self, idx, item):
         item.setParent(None)
         self.config["customSendItems"].pop(idx)
+        # TODO: here auto set scroll area height is ugly, maybe have better way
         itemsNum = self.customSendItemsLayout.count()
         height = parameters.customSendItemHeight * (itemsNum + 1) + 20
         topHeight = self.fileSendGroupBox.height() + self.logFileGroupBox.height() + 100
-        if height + topHeight > self.funcParent.height():
-            height = self.funcParent.height() - topHeight
+        # if height + topHeight > self.funcParent.height():
+        #     height = self.funcParent.height() - topHeight
+        screenH = QApplication.desktop().screenGeometry().height()
+        screenH -= screenH // 3
+        if height + topHeight >= screenH:
+            height = screenH - topHeight
         self.customSendScroll.setMinimumHeight(height)
 
     def onCustomItemChange(self, idx, edit, send):
@@ -567,7 +624,6 @@ class Plugin(Plugin_Base):
                     isHexStr, sendStr, sendStrsColored = self.bytes2String(data, not self.config["receiveAscii"], encoding=self.configGlobal["encoding"])
                     if isHexStr:
                         sendStr = sendStr.upper()
-                        sendStrsColored= sendStr
                         head += "[HEX] "
                     if self.config["useCRLF"]:
                         head = "\r\n" + head
@@ -575,9 +631,10 @@ class Plugin(Plugin_Base):
                         head = "\n" + head
                     if head.strip() != '=>':
                         head = '{}: '.format(head.rstrip())
-                    self.receiveUpdateSignal.emit(head, [sendStrsColored], self.configGlobal["encoding"])
+                    self.receiveUpdateSignal.emit(head, [sendStr], self.configGlobal["encoding"])
                     self.sendRecord.insert(0, head + sendStr)
                 self.send(data_bytes=data, callback = self.onSent)
+                self.justSent = True # flag for receive thread
                 if data_bytes:
                     data = str(data_bytes)
                 else:
@@ -788,13 +845,15 @@ class Plugin(Plugin_Base):
             buffer += new
             self.receivedData = []
             # timeout, add new line
-            if time.time() - timeLastReceive> self.config["receiveAutoLindefeedTime"]:
+            # self.justSent means just sent data, need show head
+            if time.time() - timeLastReceive > self.config["receiveAutoLindefeedTime"] / 1000 or self.justSent:
                 if self.config["showTimestamp"] or self.config["receiveAutoLinefeed"]:
                     if self.config["useCRLF"]:
                         head += "\r\n"
                     else:
                         head += "\n"
                     new_line = True
+                    self.justSent = False
             data = ""
             # have data in buffer
             if len(buffer) > 0:
@@ -808,11 +867,19 @@ class Plugin(Plugin_Base):
                 # show as string, and don't need to render color
                 elif not self.config["color"]:
                     data = buffer.decode(encoding=self.configGlobal["encoding"], errors="ignore")
+                    # self.lastShowTail for separated \r\n, prevent show two linefeed
+                    # if last msg endswith "\r", and this msg startswith "\n", don't show "\n", if you have different thought, add issue to github
+                    if self.lastShowTail == "\r" and data[0] == "\n":
+                        data = data[1:]
                     colorData = data
                     buffer = b''
+                    if data and data[-1] == "\r":
+                        self.lastShowTail = data[-1]
+                    else:
+                        self.lastShowTail = ""
                 # show as string, and need to render color, wait for \n or until timeout to ensure color flag in buffer
                 else:
-                    if time.time() - timeLastReceive >  self.config["receiveAutoLindefeedTime"] or b'\n' in buffer:
+                    if time.time() - timeLastReceive >  self.config["receiveAutoLindefeedTime"] / 1000 or b'\n' in buffer:
                         data, colorData, remain = self.getColoredText(buffer, self.configGlobal["encoding"])
                         buffer = remain
                 # add time receive head
